@@ -10,15 +10,18 @@ from typing import (
     TypeVar,
     Type,
     Coroutine,
+    Generator,
 )
-from typing_extensions import Protocol
+import enum
+from typing_extensions import Protocol, runtime_checkable
 from abc import abstractmethod
 from dataclasses import dataclass
+from contextlib import contextmanager
 import asyncio
 
 import urwid
 
-UrwidText = Union[str, Tuple[str, str], List[Tuple[str, str]]]
+UrwidText = Union[str, Tuple[str, str], List[Union[str, Tuple[str, str]]]]
 BasicType = Union[int, str, float]
 JsonType = Union[BasicType, List[BasicType], Dict[str, BasicType]]
 
@@ -66,6 +69,68 @@ class Setting(Protocol):
         pass
 
 
+@runtime_checkable
+class Device(Protocol):
+    id: str
+    name: str
+
+    @abstractmethod
+    async def readline(self) -> bytes:
+        pass
+
+    @abstractmethod
+    def write(self, data: bytes) -> None:
+        pass
+
+
+class SerialDevice(Device):
+    baudrate: int
+    path: str
+
+
+@dataclass
+class NMEA:
+    device_id: str
+    raw_message: str
+    datatype: str
+    fields: Sequence[str]
+
+
+class NMEAStream:
+    @abstractmethod
+    async def read(self) -> NMEA:
+        pass
+
+    @abstractmethod
+    def __aiter__(self):
+        pass
+
+    @abstractmethod
+    async def __anext__(self):
+        pass
+
+
+class DeviceManager(Protocol):
+    @abstractmethod
+    def register(self, device: Device) -> None:
+        pass
+
+    @abstractmethod
+    def list(self) -> List[Device]:
+        pass
+
+    @abstractmethod
+    @contextmanager
+    def open_nmea(self) -> Generator[NMEAStream, None, None]:
+        pass
+
+
+class ProcessManager(Protocol):
+    @abstractmethod
+    def start(self, coro: Coroutine) -> asyncio.Task:
+        pass
+
+
 class App(Protocol):
     name: str
     title: str
@@ -112,6 +177,17 @@ class ModalOptions:
     bottom: int = 0
 
 
+class Dialog(Protocol):
+    @abstractmethod
+    def add_button(self, label: str, handler: Callable[[], bool]) -> None:
+        pass
+
+
+class IndicatorLocation(enum.Enum):
+    LEFT = "left"
+    RIGHT = "right"
+
+
 class ScreenManager(Protocol):
     @abstractmethod
     def push_activity(
@@ -127,6 +203,20 @@ class ScreenManager(Protocol):
     def push_modal(self, activity: Activity, options: ModalOptions) -> None:
         pass
 
+    @abstractmethod
+    def push_dialog(self, title: str, content: urwid.Widget) -> Dialog:
+        pass
+
+    @abstractmethod
+    def set_indicator(
+        self, iid: str, markup: UrwidText, location: IndicatorLocation, weight: int
+    ) -> None:
+        pass
+
+    @abstractmethod
+    def remove_indicator(self, iid: str) -> None:
+        pass
+
     @abstractmethod
     def spawn_task(self, activity: Activity, coro: Coroutine) -> asyncio.Task:
         pass
@@ -164,6 +254,9 @@ class Extension(Protocol):
     def list_apps(self) -> Sequence[App]:
         return []
 
+    def start(self) -> None:
+        pass
+
 
 ExtensionFactory = Callable[[str, "OpenVarioShell"], Extension]
 
@@ -206,6 +299,8 @@ class OpenVarioShell(Protocol):
     extensions: ExtensionManager
     apps: AppManager
     os: OpenVarioOS
+    devices: DeviceManager
+    processes: ProcessManager
 
     @abstractmethod
     def quit(self) -> None:
